#!/usr/bin/env python3

import logging
import select
import textwrap
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter

from cifsdk.constants import REMOTE_ADDR, TOKEN, SEARCH_LIMIT, FORMAT, \
    COLUMNS, ADVANCED, PROFILES
from cifsdk.exceptions import AuthError, CIFBusy, InvalidSearch
from csirtg_indicator.format import FORMATS
from cifsdk.utils import setup_logging, get_argument_parser
from csirtg_indicator import Indicator
from cifsdk.client.http import HTTP as Client
from pprint import pprint


logger = logging.getLogger(__name__)


def _search(cli, args, options, filters):
    fmt = options.get('format')
    if args.profile:
        for k, v in PROFILES[args.profile].items():
            if k == 'format':
                fmt = v
            else:
                if not filters.get(k):
                    filters[k] = v

    if not filters.get('indicator') and (not filters.get('itype') and
                                         ADVANCED is False):
        print('\nmissing --itype\n\n')
        raise SystemExit

    if not filters.get('indicator') and (not filters.get('tags') and
                                         ADVANCED is False):
        print('\nmissing --tags [phishing|malware|botnet|scanner|pdns|'
              'whitelist|...]\n\n')
        raise SystemExit

    try:
        rv = cli.search(filters)

    except AuthError as e:
        logger.error('unauthorized')

    except CIFBusy as e:
        print("\ncif-router is either too busy or requires a restart...")
        print("if the problem continues, try increasing the amount of memory "
              "and cpus for the system")
        print("\nhttps://github.com/csirtgadgets/verbose-robot/wiki/FAQ\n"
              "https://csirtg.io/support\n")
        raise SystemExit

    except KeyboardInterrupt:
        pass

    except InvalidSearch as e:
        if logger.getEffectiveLevel() == logging.DEBUG:
            logger.error(e)

        else:
            print(f"\n{e}\n")

    except Exception as e:
        import traceback
        traceback.print_exc()
        logger.error(e)

    else:

        for l in FORMATS[fmt](data=sorted(rv, key=lambda i: i['reported_at']), cols=args.columns.split(',')):
            print(l)

    raise SystemExit


def _graph(cli, args, options, filters):
    try:
        rv = cli.graph_search(filters)

    except AuthError as e:
        logger.error('unauthorized')

    except KeyboardInterrupt:
        pass

    except Exception as e:
        import traceback
        traceback.print_exc()
        logger.error(e)

    else:
        pprint(rv['data'])

    raise SystemExit


def _ping(cli, args):
    logger.info('running ping')
    n = 4
    if args.ping_indef:
        n = 999

    try:
        for num in range(0, n):
            ret = cli.ping()
            if ret != 0:
                print("roundtrip: {} ms".format(ret))
                select.select([], [], [], 1)
                from time import sleep
                sleep(1)
            else:
                logger.error('ping failed')
                raise RuntimeError
    except KeyboardInterrupt:
        pass
    raise SystemExit


def _submit(cli, args, options):
    print("submitting {0}".format(options.get("submit")))
    i = Indicator(indicator=args.indicator, tags=args.tags,
                  confidence=args.confidence)

    rv = cli.indicators_create(i)

    print('success id: {}\n'.format(rv))
    raise SystemExit


def _delete(cli, args, filters):
    if args.id:
        filters = {'id': args.id}

    filters = {f: filters[f] for f in filters if filters.get(f)}
    print("deleting {0}".format(filters))
    rv = cli.indicators_delete(filters)

    print('deleted: {}'.format(rv))
    raise SystemExit


def main():
    p = get_argument_parser()
    p = ArgumentParser(
        description=textwrap.dedent('''\
        example usage:
            $ cif -q example.org -d
            $ cif --search 1.2.3.0/24
            $ cif --ping
        '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='cif',
        parents=[p]
    )
    p.add_argument('--token', help='specify api token', default=TOKEN)
    p.add_argument('--remote', help='specify API remote [default %(default)s]', default=REMOTE_ADDR)
    p.add_argument('--no-verify-ssl', action='store_true')

    p.add_argument('-p', '--ping', action="store_true")
    p.add_argument('--ping-indef', action="store_true")
    p.add_argument("--submit", action="store_true", help="submit an indicator")
    p.add_argument('--delete', action='store_true')
    p.add_argument('-q', '--search', help="search")

    p.add_argument('--id')
    p.add_argument('--itype', help='filter by indicator type')
    p.add_argument('--reported_at', help='specify reported_at filter')
    p.add_argument('-n', '--nolog', help='do not log search', action='store_true')
    p.add_argument('-f', '--format', help='specify output format [default: %(default)s]"',
                   default=FORMAT, choices=FORMATS.keys())
    p.add_argument('--indicator')
    p.add_argument('--confidence', help="specify confidence level")
    p.add_argument('--probability')
    p.add_argument('--tags', nargs='+')
    p.add_argument('--provider')
    p.add_argument('--asn')
    p.add_argument('--cc')
    p.add_argument('--asn-desc')
    p.add_argument('--rdata')
    p.add_argument('--region')
    p.add_argument('--groups', help='specify groups filter (csv)')

    p.add_argument('--days', help='filter results within last X days')
    p.add_argument('--today', help='auto-sets reporttime to today, 00:00:00Z (UTC)', action='store_true')

    p.add_argument('--limit', help='limit results [default %(default)s]', default=SEARCH_LIMIT)
    p.add_argument('--columns', help='specify output columns [default %(default)s]', default=','.join(COLUMNS))
    p.add_argument('--no-feed', action='store_true')

    p.add_argument('--graph', help='dump the graph', action='store_true')

    p.add_argument('--profile', help='specify feed profile',
                   choices=PROFILES.keys())

    args = p.parse_args()

    setup_logging(args)

    opts = vars(args)

    options = {}
    for k, v in opts.items():
        if v:
            options[k] = v

    verify_ssl = True
    if args.no_verify_ssl:
        verify_ssl = False

    if args.remote == 'https://localhost':
        verify_ssl = False

    cli = Client(args.remote, args.token, verify_ssl=verify_ssl)

    if args.ping or args.ping_indef:
        _ping(cli, args)

    filters = {
        'itype': options.get('itype'),
        'limit': options.get('limit'),
        'provider': options.get('provider'),
        'indicator': options.get('search') or options.get('indicator'),
        'nolog': options.get('nolog'),
        'tags': options.get('tags'),
        'confidence': options.get('confidence'),
        'asn': options.get('asn'),
        'asn_desc': options.get('asn_desc'),
        'cc': options.get('cc'),
        'region': options.get('region'),
        'rdata': options.get('rdata'),
        'reported_at': options.get('reported_at'),
        'groups': options.get('groups'),
        'hours': options.get('hours'),
        'days': options.get('days'),
        'today': options.get('today'),
        'no_feed': options.get('no_feed'),
        'probability': options.get('probability')
    }

    for k, v in filters.items():
        if v is True:
            filters[k] = 1
        if v is False:
            filters[k] = 0

    if options.get("submit"):
        _submit(cli, args, options)

    if options.get("delete"):
        _delete(cli, args, options, filters)

    if args.graph:
        _graph(cli, args, options, filters)

    _search(cli, args, options, filters)


if __name__ == "__main__":
    main()



from ._version import get_versions
VERSION = get_versions()['version']
del get_versions

import os.path
import tempfile
import sys
from csirtg_indicator.constants import COLUMNS

PYVERSION = 2
if sys.version_info > (3,):
    PYVERSION = 3

TEMP_DIR = os.path.join(tempfile.gettempdir())
RUNTIME_PATH = os.getenv('CIF_RUNTIME_PATH', TEMP_DIR)
DATA_PATH = os.getenv('CIF_DATA_PATH', TEMP_DIR)


# Logging stuff
LOG_FORMAT = '%(asctime)s - %(levelname)s - %(name)s[%(lineno)s][%(threadName)s] - %(message)s'

LOGLEVEL = 'ERROR'
LOGLEVEL = os.getenv('CIF_LOGLEVEL', LOGLEVEL).upper()

CONFIG_PATH = os.getenv('CIF_CONFIG_PATH', os.path.join(os.getcwd(), 'cif.yml'))
if not os.path.isfile(CONFIG_PATH):
    CONFIG_PATH = os.path.join(os.path.expanduser('~'), '.cif.yml')

# address stuff

REMOTE_ADDR = 'http://localhost:5000'
REMOTE_ADDR = os.getenv('CIF_REMOTE', REMOTE_ADDR)

ROUTER_ADDR = "ipc://{}".format(os.path.join(RUNTIME_PATH, 'router.ipc'))
ROUTER_ADDR = os.getenv('CIF_ROUTER_ADDR', ROUTER_ADDR)

SEARCH_LIMIT = os.getenv('CIF_SEARCH_LIMIT', 500)

TOKEN = os.getenv('CIF_TOKEN', None)
FORMAT = os.getenv('CIF_FORMAT', 'table')

ADVANCED = os.getenv('CIF_ADVANCED', False)
if ADVANCED == '1':
    ADVANCED = True
else:
    ADVANCED = False

COLUMNS = os.getenv('CIFSDK_COLUMNS', COLUMNS)
if not isinstance(COLUMNS, list):
    COLUMNS = COLUMNS.split(',')


MAX_FIELD_SIZE = 30

VALID_FILTERS = ['indicator', 'itype', 'confidence', 'provider', 'limit', 'application', 'nolog', 'tags', 'days',
                 'hours', 'groups', 'reported_at', 'cc', 'asn', 'asn_desc', 'rdata', 'first_at', 'last_at', 'region',
                 'probability', 'no_feed', 'days', 'hours', 'today', 'latitude', 'longitude', 'probability']

PROFILES = {
    'splunk': {
        'format': 'csv',
        'confidence': 3,
        'hours': 1,
        'limit': 25000,
        'itype': 'ipv4',
    },
    'bind': {
        'format': 'bind',
        'confidence': 4,
        'itype': 'fqdn',
        'days': 45,
        'tags': 'phishing,malware,botnet',
        'limit': 250000,
    },
    'bro': {
        'confidence': 3,
        'format': 'bro',
        'itype': 'ipv4',
        'days': 21,
        'limit': 250000,
        'tags': 'botnet'
    },
    'snort': {
        'confidence': 3,
        'itype': 'ipv4',
        'format': 'snort',
        'days': 21,
        'limit': 250000,
        'tags': 'botnet'
    },
    'firewall': {
        'format': 'csv',
        'itype': 'ipv4',
        'confidence': 4,
        'days': 21,
        'tags': 'scanner,bruteforce,botnet',
        'limit': 25000,
    },
    'sem': {
        'format': 'csv',
        'confidence': 3,
        'hours': 1,
        'limit': 25000,
        'itype': 'ipv4',
    },
}


import ujson as json
from pprint import pprint
import msgpack
import sys
import logging
import os

try:
    import zmq
except ImportError:
    print("Requires pyzmq")
    raise SystemExit

TRACE = os.getenv('CIFSDK_MSG_TRACE', False)

logger = logging.getLogger(__name__)
logger.setLevel(logging.ERROR)

if TRACE == '1':
    logger.setLevel(logging.DEBUG)

PYVERSION = 2
if sys.version_info > (3,):
    PYVERSION = 3

MAP = {
    1: 'ping',
    2: 'ping_write',
    3: 'indicators_create',
    4: 'indicators_search',
    5: 'indicators_delete',
    6: 'tokens_search',
    7: 'tokens_create',
    8: 'tokens_delete',
    9: 'tokens_edit',
    10: 'graph_search',
    11: 'stats_search',
}


class Msg(object):

    PING = 1
    PING_WRITE = 2
    INDICATORS_CREATE = 3
    INDICATORS_SEARCH = 4
    INDICATORS_DELETE = 5
    TOKENS_SEARCH = 6
    TOKENS_CREATE = 7
    TOKENS_DELETE = 8
    TOKENS_EDIT = 9
    GRAPH_SEARCH = 10
    STATS_SEARCH = 11

    def __init__(self, *args, **kwargs):
        for k in kwargs:
            if isinstance(kwargs[k], str):
                try:
                    kwargs[k] = kwargs[k].encode('utf-8')
                except UnicodeDecodeError:
                    pass

        self.id = kwargs.get('id')
        self.client_id = kwargs.get('client_id')
        self.mtype = kwargs.get('mtype')
        self.token = kwargs.get('token')
        self.data = kwargs.get('data')
        self.null = ''.encode('utf-8')

    def __repr__(self):
        m = {
            'id': self.id,
            'mtype': self.mtype,
            'token': self.token,
            'data': self.data,
        }

        return json.dumps(m)

    def mtype_to_int(self, mtype):
        for m in MAP:
            if MAP[m] == mtype:
                return m

    def send(self, s):
        # assert isinstance(s, zmq.socket)
        m = self.to_list()
        logger.debug(m)
        s.send_multipart(m)

    def recv(self, s, relay=False):
        # assert isinstance(s, zmq.socket)
        m = s.recv_multipart()

        if relay:
            assert isinstance(relay, zmq.Socket)
            return relay.send_multipart(m)

        return self._decode_message(m)

    def _decode_message(self, m):

        # zmq has diff frame types depending on
        if len(m) == 6:
            id, client_id, null, token, mtype, data = m
            mtype = msgpack.unpackb(mtype)
            mtype = MAP[mtype]
            return id, client_id, token.decode('utf-8'), mtype, data.decode('utf-8')

        if len(m) == 5:
            id, null, token, mtype, data = m
            mtype = msgpack.unpackb(mtype)
            mtype = MAP[mtype]
            return id, token.decode('utf-8'), mtype, data.decode('utf-8')

        if len(m) == 4:
            id, token, mtype, data = m
            mtype = msgpack.unpackb(mtype)
            mtype = MAP[mtype]
            return id, token.decode('utf-8'), mtype, data.decode('utf-8')

        if len(m) == 3:
            id, mtype, data = m
            try:
                mtype = msgpack.unpackb(mtype)
                mtype = MAP[mtype]
            except msgpack.exceptions.ExtraData:
                pass
            return id, mtype, data.decode('utf-8')

        mtype, data = m
        return mtype, data.decode("utf-8")

    def to_list(self):
        m = []
        if self.id:
            m.append(self.id)

        if self.client_id:
            m.append(self.client_id)

        if len(m) > 0:
            m.append(self.null)

        if self.token:
            if isinstance(self.token, str):
                self.token = self.token.encode('utf-8')

            if PYVERSION == 2:
                if isinstance(self.token, unicode):
                    self.token = self.token.encode('utf-8')

            m.append(self.token)

        if self.mtype:
            if isinstance(self.mtype, bytes):
                self.mtype = self.mtype_to_int(self.mtype.decode('utf-8'))

            m.append(msgpack.packb(self.mtype))

        if isinstance(self.data, dict):
            self.data = [self.data]

        if isinstance(self.data, list):
            self.data = json.dumps(self.data)

        # change this to feature detection try/except
        if isinstance(self.data, str):
            self.data = self.data.encode('utf-8')

        if PYVERSION == 2:
            if isinstance(self.data, unicode):
                self.data = self.data.encode('utf-8')

        m.append(self.data)
        return m



import json
import logging
import os
import zlib
from zmq.eventloop.ioloop import IOLoop
from zmq.eventloop.zmqstream import ZMQStream
from pprint import pprint

try:
    import zmq
except ImportError:
    print("Requires pyzmq")
    raise SystemExit

from cifsdk.client import Client
from cifsdk.msg import Msg
from cifsdk.exceptions import AuthError, TimeoutError, InvalidSearch, CIFBusy
from cifsdk.constants import PYVERSION
from csirtg_indicator import Indicator

SNDTIMEO = os.getenv('ZMQ_SNDTIMEO', '90000')
SNDTIMEO = int(SNDTIMEO)
RCVTIMEO = os.getenv('ZMQ_RCVTIMEO', '90000')
RCVTIMEO = int(RCVTIMEO)
LINGER = 3
ENCODING_DEFAULT = "utf-8"
SEARCH_LIMIT = 100
FIREBALL_SIZE = os.getenv('CIFSDK_CLIENT_ZMQ_FIREBALL_SIZE', 500)
FIREBALL_SIZE = int(FIREBALL_SIZE)
TRACE = os.getenv('CIFSDK_CLIENT_ZMQ_TRACE')

logger = logging.getLogger(__name__)

logger.setLevel(logging.ERROR)

if TRACE:
    logger.setLevel(logging.DEBUG)

if PYVERSION == 3:
    basestring = (str, bytes)


class ZMQ(Client):
    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):

        # do our best to clean up potentially leaky FDs
        if hasattr(self, 'stream') and not self.stream.closed():
            self.stream.close()

        if hasattr(self, 'loop'):
            try:
                self.loop.close()
            except KeyError:
                pass

        self.socket.close()
        self.context.term()

    def __init__(self, remote, token, **kwargs):
        super(ZMQ, self).__init__(remote, token)

        self.context = zmq.Context()
        self.socket = self.context.socket(zmq.REQ)
        self.socket.RCVTIMEO = RCVTIMEO
        self.socket.SNDTIMEO = SNDTIMEO
        self.socket.setsockopt(zmq.LINGER, LINGER)
        self.nowait = kwargs.get('nowait', False)
        self.autoclose = kwargs.get('autoclose', True)
        if self.nowait:
            self.socket = self.context.socket(zmq.DEALER)

        self.autoclose = kwargs.get('autoclose', True)

    def _handle_message_fireball(self, m):
        logger.debug('message received')

        m = json.loads(m[2].decode('utf-8'))
        self.response.append(m)

        self.num_responses -= 1
        logger.debug('num responses remaining: %i' % self.num_responses)

        if self.num_responses == 0:
            logger.debug('finishing up...')
            self.loop.stop()

    def _fireball_timeout(self):
        logger.info('fireball timeout')
        self.loop.stop()

    def _send_fireball(self, mtype, data, f_size):
        if len(data) < 3:
            logger.error('no data to send')
            return []

        self.loop = IOLoop().instance()
        self.socket.close()

        self.socket = self.context.socket(zmq.DEALER)
        self.socket.connect(self.remote)

        self.stream = ZMQStream(self.socket)
        self.stream.on_recv(self._handle_message_fireball)

        self.stream.io_loop.call_later(SNDTIMEO, self._fireball_timeout)

        self.response = []

        if PYVERSION == 3:
            if isinstance(data, bytes):
                data = data.decode('utf-8')

        data = json.loads(data)

        if not isinstance(data, list):
            data = [data]

        if (len(data) % f_size) == 0:
            self.num_responses = int((len(data) / f_size))
        else:
            self.num_responses = int((len(data) / f_size)) + 1

        logger.debug('responses expected: %i' % self.num_responses)

        batch = []
        for d in data:
            batch.append(d)
            if len(batch) == f_size:
                Msg(mtype=Msg.INDICATORS_CREATE, token=self.token, data=batch).send(self.socket)
                batch = []

        if len(batch):
            Msg(mtype=Msg.INDICATORS_CREATE, token=self.token, data=batch).send(self.socket)

        logger.debug("starting loop to receive")
        self.loop.start()

        # clean up FDs
        self.loop.close()
        self.stream.close()
        self.socket.close()
        return self.response

    def _recv(self, decode=True, close=True):
        mtype, data = Msg().recv(self.socket)
        if close:
            self.socket.close()

        if not decode:
            return data

        data = json.loads(data)

        if data.get('message') == 'unauthorized':
            raise AuthError()

        if data.get('message') == 'busy':
            raise CIFBusy()

        if data.get('message') == 'invalid search':
            raise InvalidSearch()

        if data.get('status') != 'success':
            raise RuntimeError(data.get('message'))

        if data.get('data') is None:
            raise RuntimeError('invalid response')

        if isinstance(data.get('data'), bool):
            return data['data']

        # is this a straight up elasticsearch string?
        if data['data'] == '{}':
            return []

        if isinstance(data['data'], basestring) and data['data'].startswith('{"hits":{"hits":[{"_source":'):
            data['data'] = json.loads(data['data'])
            data['data'] = [r['_source'] for r in data['data']['hits']['hits']]

        try:
            data['data'] = zlib.decompress(data['data'])
        except (zlib.error, TypeError):
            pass

        return data.get('data')

    def _send(self, mtype, data='[]', nowait=False, decode=True):

        self.socket.connect(self.remote)

        if isinstance(data, str):
            data = data.encode('utf-8')

        Msg(mtype=mtype, token=self.token, data=data).send(self.socket)

        if self.nowait or nowait:
            if self.autoclose:
                self.socket.close()
            return

        rv = self._recv(decode=decode)
        return rv

    def ping(self):
        try:
            return self._send(Msg.PING)
        except zmq.error.Again:
            raise TimeoutError

    def ping_write(self):
        try:
            return self._send(Msg.PING_WRITE)
        except zmq.error.Again:
            raise TimeoutError

    def indicators_search(self, filters, decode=True):
        return self._send(Msg.INDICATORS_SEARCH, json.dumps(filters), decode=decode)

    def graph_search(self, filters, decode=True):
        return self._send(Msg.GRAPH_SEARCH, json.dumps(filters), decode=decode)

    def stats_search(self, filters, decode=True):
        return self._send(Msg.STATS_SEARCH, json.dumps(filters), decode=decode)

    def indicators_create(self, data, nowait=False, fireball=False, f_size=FIREBALL_SIZE):
        if isinstance(data, dict):
            data = self._kv_to_indicator(data)

        if isinstance(data, Indicator):
            data = str(data)

        if fireball:
            return self._send_fireball(Msg.INDICATORS_CREATE, data, f_size)

        return self._send(Msg.INDICATORS_CREATE, data, nowait=nowait)

    def indicators_delete(self, data):
        if isinstance(data, dict):
            data = self._kv_to_indicator(data)

        if isinstance(data, Indicator):
            data = str(data)

        return self._send(Msg.INDICATORS_DELETE, data)

    def tokens_search(self, filters={}):
        return self._send(Msg.TOKENS_SEARCH, json.dumps(filters))

    def tokens_create(self, data):
        return self._send(Msg.TOKENS_CREATE, data)

    def tokens_delete(self, data):
        return self._send(Msg.TOKENS_DELETE, data)

    def tokens_edit(self, data):
        return self._send(Msg.TOKENS_EDIT, data)



import websocket
import logging
import os
import json
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
import textwrap
from csirtgsdk.constants import LOG_FORMAT
from websocket import create_connection

try:
    import thread
except ImportError:
    import _thread as thread

REMOTE = os.getenv('CIF_REMOTE_WS', 'ws://localhost:5000/firehose')

logger = logging.getLogger(__name__)

TRACE = os.environ.get('CIFSDK_CLIENT_WS_TRACE', '0')

if TRACE == '1':
    logger.setLevel(logging.DEBUG)
    logging.getLogger('websocket').setLevel(logging.DEBUG)
    websocket.enableTrace(True)

TOKEN = os.getenv('CIF_TOKEN')


class DefaultHandler(websocket.WebSocket):
    def __init__(self, remote=REMOTE, token=TOKEN, **kvargs):

        self.handle = websocket.WebSocketApp(
            remote,
            on_message=self.on_message,
            on_error=self.on_error,
            on_close=self.on_close,
            on_open=self.on_open,
            header={'Authorization: ' + token},
        )

        self.error = False

    def on_message(self, ws, message):
        logger.debug(message)

        if isinstance(message, bytes):
            message = message.decode('utf8')

        try:
            message = json.loads(message)
        except json.JSONDecodeError:
            pass

        if message == 'ping':
            self.handle.send('pong')
            return

        if message == 'connected' and logger.getEffectiveLevel() == logging.DEBUG:
            print('connected...')
            return

        if logger.getEffectiveLevel() == logging.DEBUG:
            print(json.dumps(message, indent=4, sort_keys=True, separators=(',', ': ')))
        else:
            print(json.dumps(message))

    def on_error(self, ws, error):
        if len(str(error)):
            logger.error(error)
            self.error = True

    def connected(self, ws, **kwargs):
        logger.info('connected...')

    def on_close(self, ws):
        logger.info(' ### CLOSED ###')

    def on_open(self, ws):
        pass

    def run(self):
        self.handle.run_forever()


def main():
    class MyHandler(DefaultHandler):
        def on_close(self, ws):
            ws.close()
            logger.info("Closed using MyHandler...")

    parser = ArgumentParser(
        description=textwrap.dedent('''\
            example usage:
              $ export CIF_TOKEN=abcdefg1234...
              $ cif-firehose -v
            '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='cif-firehose'
    )

    parser.add_argument("-v", "--verbose", action="count", help="set verbosity level to INFO")
    parser.add_argument('-d', '--debug', action="store_true")
    parser.add_argument('-r', '--reconnect', action="store_true", help="auto reconnect if connection closes...", default=False)

    args = parser.parse_args()

    loglevel = logging.WARNING
    if args.verbose:
        loglevel = logging.INFO
    if args.debug:
        loglevel = logging.DEBUG

    logger.setLevel(loglevel)

    console = logging.StreamHandler()
    logging.getLogger('').setLevel(loglevel)
    console.setFormatter(logging.Formatter(LOG_FORMAT))
    logging.getLogger('').addHandler(console)

    h = MyHandler()

    while True:
        h.run()

        # if we closed and it wasn't an error
        if not h.error:
            break

        # if we didn't set the re-connect flag
        if not args.reconnect:
            break

        # we set the re-connect and it was an error, clear the error and re-connect.
        h.error = False

        logger.info('re-connecting..')


if __name__ == "__main__":
    main()



from cifsdk.client.dummy import Dummy as Client


def test_dummy():
    cli = Client('https://localhost:3000', 12345)
    assert cli.remote == 'https://localhost:3000'

    data = {
        'indicator': 'example.com',
        'tags': ['botnet']
    }
    assert cli.indicators_create(data)

    assert cli.indicators_search(data)



import pkgutil
import logging
from cifsdk.constants import LOG_FORMAT, RUNTIME_PATH, LOGLEVEL, VERSION
from argparse import ArgumentParser
import signal
from . import color

import yaml
import os


def settings(f):
    if not os.path.exists(f):
        return

    with open(f, "r") as f:
        return yaml.load(f)


def read_config(args):
    options = {}
    if os.path.isfile(args.config):
        with open(args.config) as f:
            config = yaml.load(f)

        if not config:
            return options

        if config.get('client'):
            config = config['client']
        f.close()
        if not config:
            print("Unable to read {} config file".format(args.config))
            raise SystemExit
        for k in config:
            if not options.get(k):
                options[k] = config[k]
    else:
        print("Unable to read {} config file".format(args.config))
        raise SystemExit

    return options


def get_argument_parser():
    BasicArgs = ArgumentParser(add_help=False)
    BasicArgs.add_argument('-d', '--debug', dest='debug', action="store_true")
    BasicArgs.add_argument('-v', '--verbose', action='store_true')
    BasicArgs.add_argument('-V', '--version', action='version', version=VERSION)
    BasicArgs.add_argument(
        "--runtime-path", help="specify the runtime path [default %(default)s]", default=RUNTIME_PATH
    )
    return ArgumentParser(parents=[BasicArgs], add_help=False)


def load_plugin(path, plugin):
    p = None
    for loader, modname, is_pkg in pkgutil.iter_modules(path):
        if modname == plugin or modname == 'z{}'.format(plugin):
            p = loader.find_module(modname).load_module(modname)
            p = p.Plugin

    return p


def load_plugins(path):
    p = []
    for loader, modname, is_pkg in pkgutil.iter_modules(path):
        p.append(loader.find_module(modname).load_module(modname))

    return p


def setup_logging(args):
    loglevel = logging.getLevelName(LOGLEVEL)

    if args.debug:
        loglevel = logging.DEBUG

    console = logging.StreamHandler()
    logging.getLogger('').setLevel(loglevel)
    console.setFormatter(logging.Formatter(LOG_FORMAT))
    logging.getLogger('').addHandler(console)


def setup_signals(name):
    logger = logging.getLogger(__name__)

    def sigterm_handler(_signo, _stack_frame):
        logger.info('SIGTERM Caught for {}, shutting down...'.format(name))
        raise SystemExit

    signal.signal(signal.SIGTERM, sigterm_handler)


def setup_runtime_path(path):
    if not os.path.isdir(path):
        os.mkdir(path)



from cifsdk.client import Client
import ujson as json


class Dummy(Client):

    def __init__(self, remote, token):
        self.remote = remote
        self.token = token

    def ping(self, write=False):
        return True

    def indicators_create(self, data):
        if isinstance(data, dict):
            data = self._kv_to_indicator(data)

        if type(data) == dict:
            data = [data]

        return data

    def indicators_search(self, data, decode=True, test_data=[], test_wl=[]):
        if len(test_data) > 0:
            return test_data

        if type(data) == dict:
            data = [data]

        if decode:
            return data

        return json.dumps({'data': data, 'status': 'success', 'message': ''}).encode('utf-8')


Plugin = Dummy



import logging
import requests
import time
import json
from pprint import pprint
from base64 import b64decode
import os
from time import sleep
import random

from cifsdk.exceptions import AuthError, TimeoutError, NotFound, SubmissionFailed, InvalidSearch, CIFBusy
from cifsdk.constants import VERSION, PYVERSION
from cifsdk.client import Client
from csirtg_indicator import Indicator

TRACE = os.environ.get('CIFSDK_CLIENT_HTTP_TRACE')
TIMEOUT = os.getenv('CIFSDK_CLIENT_HTTP_TIMEOUT', 120)
RETRIES = os.getenv('CIFSDK_CLIENT_HTTP_RETRIES', 5)
RETRIES_DELAY = os.getenv('CIFSDK_CLIENT_HTTP_RETRIES_DELAY', '30,60')

REMOTE = os.getenv('CIF_REMOTE', 'http://localhost:5000')
TOKEN = os.getenv('CIF_TOKEN')

if PYVERSION == 3:
    basestring = (str, bytes)

s, e = RETRIES_DELAY.split(',')
RETRIES_DELAY = random.uniform(int(s), int(e))

requests.packages.urllib3.disable_warnings()

logger = logging.getLogger(__name__)

logger.setLevel(logging.WARNING)
logging.getLogger('requests.packages.urllib3.connectionpool').setLevel(logging.ERROR)

if TRACE:
    logger.setLevel(logging.DEBUG)
    logging.getLogger('requests.packages.urllib3.connectionpool').setLevel(logging.DEBUG)


class HTTP(Client):

    def __init__(self, remote=REMOTE, token=TOKEN, proxy=None, timeout=int(TIMEOUT), verify_ssl=True, **kwargs):
        if remote is None:
            remote = 'http://localhost:5000'

        if token is None:
            token = os.getenv('CIF_TOKEN')

        super(HTTP, self).__init__(remote, token, **kwargs)

        self.proxy = proxy
        self.timeout = timeout
        self.verify_ssl = verify_ssl
        self.nowait = kwargs.get('nowait', False)

        self.session = requests.Session()
        self.session.headers["Accept"] = 'application/vnd.cif.v4+json'
        self.session.headers['User-Agent'] = 'cifsdk-py/{}'.format(VERSION)
        self.session.headers['Authorization'] = self.token
        self.session.headers['Accept-Encoding'] = 'deflate'

    def _check_status(self, resp, expect=200):
        if resp.status_code == expect:
            return

        if resp.status_code == 400:
            r = json.loads(resp.text)
            raise InvalidSearch(r['message'])

        if resp.status_code == 401:
            raise AuthError('unauthorized')

        if resp.status_code == 404:
            raise NotFound('not found')

        if resp.status_code == 408:
            raise TimeoutError('timeout')

        if resp.status_code == 422:
            msg = json.loads(resp.text)
            raise SubmissionFailed(msg['message'])

        if resp.status_code == 429:
            raise CIFBusy('RateLimit exceeded')

        if resp.status_code in [500, 501, 502, 503, 504]:
            raise CIFBusy('system seems busy..')

        if resp.status_code != expect:
            msg = 'unknown: %s' % resp.content
            raise RuntimeError(msg)

    def __get(self, uri, params):
        resp = self.session.get(uri, params=params, verify=self.verify_ssl, timeout=self.timeout)
        logger.debug(resp.text)
        n = RETRIES
        try:
            self._check_status(resp, expect=200)
            n = 0
            return resp
        except Exception as e:
            if resp.status_code == 429 or resp.status_code in [500, 501, 502, 503, 504]:
                logger.error(e)
            else:
                raise e

        for nn in (1, n):
            logger.warning('setting random retry interval to spread out the load')
            logger.warning('retrying in %.00fs' % RETRIES_DELAY)
            sleep(RETRIES_DELAY)

            resp = self.session.get(uri, params=params, verify=self.verify_ssl, timeout=self.timeout)
            if resp.status_code == 200:
                break

            if nn == n:
                raise CIFBusy('system seems busy.. try again later')

        return resp

    def _check_data(self, msgs):
        if isinstance(msgs, list):
            return msgs

        if msgs.get('status', False) not in ['success', 'failure']:
            raise RuntimeError(msgs)

        if msgs.get('status') == 'failure':
            raise InvalidSearch(msgs['message'])

        if msgs['data'] == '{}':
            msgs['data'] = []

        # check to see if it's straight elasticsearch json
        if isinstance(msgs['data'], basestring) and msgs['data'].startswith('{"hits":{"hits":[{"_source":'):
            msgs['data'] = json.loads(msgs['data'])
            msgs['data'] = [r['_source'] for r in msgs['data']['hits']['hits']]

        if not isinstance(msgs['data'], list):
            msgs['data'] = [msgs['data']]

        for m in msgs['data']:
            if not isinstance(m, dict):
                continue

            if not m.get('message'):
                continue

            try:
                m['message'] = b64decode(m['message'])
            except Exception as e:
                pass

        return msgs

    def _get(self, uri, params={}, retry=True):
        if not uri.startswith('http'):
            uri = "%s/%s/" % (self.remote, uri)

        resp = self.__get(uri, params)

        data = resp.content

        ss = (int(resp.headers['Content-Length']) / 1024 / 1024)
        logger.info('processing %.2f megs' % ss)

        msgs = json.loads(data.decode('utf-8'))

        msgs = self._check_data(msgs)
        return msgs

    def _post(self, uri, data, expect=201):
        if not uri.startswith('http'):
            uri = "%s/%s/" % (self.remote, uri)

        data = json.dumps(data)

        if self.nowait:
            uri = '{}?nowait=1'.format(uri)

        if isinstance(data, str):
            data = data.encode('utf-8')

        headers = {'Content-Type': 'application/json'}
        logger.debug('submitting')
        resp = self.session.post(uri, data=data, verify=self.verify_ssl,
                                 headers=headers, timeout=self.timeout)
        logger.debug(resp.content)
        logger.debug(resp.status_code)
        n = RETRIES
        try:
            self._check_status(resp, expect=expect)
            n = 0
        except Exception as e:
            if resp.status_code == 429 or resp.status_code in [500, 501, 502,
                                                               503, 504]:
                logger.error(e)
            else:
                raise e

        while n != 0:
            logger.info('setting random retry interval to spread out the load')
            logger.info('retrying in %.00fs' % RETRIES_DELAY)
            sleep(RETRIES_DELAY)

            resp = self.session.post(uri, data=data, verify=self.verify_ssl,
                                     headers=headers, timeout=self.timeout)
            if resp.status_code in [200, 201]:
                break

            if n == 0:
                raise CIFBusy('system seems busy.. try again later')

        return json.loads(resp.content.decode('utf-8'))

    def _delete(self, uri, params={}):
        if not uri.startswith('http'):
            uri = "%s/%s/" % (self.remote, uri)

        params = {f: params[f] for f in params if params.get(f)}

        for f in ['nolog', 'limit']:
            if params.get(f):
                del params[f]

        resp = self.session.delete(uri, data=json.dumps(params), verify=self.verify_ssl, timeout=self.timeout)
        self._check_status(resp)
        return json.loads(resp.content.decode('utf-8'))

    def _patch(self, uri, data):
        if not uri.startswith('http'):
            uri = "%s/%s/" % (self.remote, uri)

        resp = self.session.patch(uri, data=json.dumps(data))
        self._check_status(resp)
        return json.loads(resp.content)

    def ping(self, write=False):
        t0 = time.time()
        rv = self._get('ping')
        if not rv:
            return

        rv = (time.time() - t0)
        logger.debug('return time: %.15f' % rv)
        return rv

    def ping_write(self):
        t0 = time.time()
        rv = self._post('ping', data=[], expect=200)
        if not rv:
            return

        rv = (time.time() - t0)
        logger.debug('return time: %.15f' % rv)
        return rv

    def graph_search(self, filters):
        return self._get('graph', params=filters)

    def stats_search(self, filters={}):
        return self._get('stats', params=filters)

    def indicators_search(self, filters):
        data = self._get('indicators', params=filters)
        return data

    def indicators_search_bulk(self, data):
        return self._post('indicators/bulk', data, 200)

    def indicators_create(self, data):
        if isinstance(data, Indicator):
            data = data.__dict__()

        if type(data) == dict:
            data = [data]

        if isinstance(data, list) and isinstance(data[0], Indicator):
            data = [i.__dict__() for i in data]

        rv = self._post('indicators', data)
        return rv["data"]

    def indicators_delete(self, filters):
        rv = self._delete('indicators', params=filters)
        return rv["data"]

    def tokens_search(self, filters):
        return self._get('tokens', params=filters)

    def tokens_delete(self, data):
        return self._delete('tokens', data)

    def tokens_create(self, data):
        return self._post('tokens', data)

    def tokens_edit(self, data):
        return self._patch('tokens', data)


Plugin = HTTP



import os
from setuptools import setup, find_packages
import versioneer
import sys

if sys.version_info < (3, 6):
    print("\n")
    print("This requires python 3.6 or higher")
    print("\n")
    raise SystemExit


# https://www.pydanny.com/python-dot-py-tricks.html
if sys.argv[-1] == 'test':
    test_requirements = [
        'pytest',
        'coverage',
        'pytest_cov',
    ]
    try:
        modules = map(__import__, test_requirements)
    except ImportError as e:
        err_msg = e.message.replace("No module named ", "")
        msg = "%s is not installed. Install your test requirements." % err_msg
        raise ImportError(msg)
    r = os.system('py.test test -v --cov=cifsdk --cov-fail-under=30 --pep8')
    if r == 0:
        sys.exit()
    else:
        raise RuntimeError('tests failed')

setup(
    name="cifsdk",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="CIFv4 Python SDK",
    long_description="",
    url="https://github.com/csirtgadgets/verbose-robot-sdk-py",
    license='MPL2',
    classifiers=[
               "Topic :: System :: Networking",
               "Programming Language :: Python",
               ],
    keywords=['security'],
    author="Wes Young",
    author_email="wes@csirtgadgets.com",
    packages=find_packages(exclude=("tests",)),
    install_requires=[
        'ujson',
        'msgpack',
        'tornado',
        'csirtg_indicator>=2.0a24,<3.0',
        'requests',
        'pyyaml',
        'websocket-client',
        'geoip2',
        'csirtgsdk',
        'tornado'

    ],
    scripts=[],
    entry_points={
        'console_scripts': [
            'cif=cifsdk.cli:main',
            'cif-tokens=cifsdk.tokens:main',
            'cif-firehose=cifsdk.client.ws:main'
        ]
    },
)
